@@ -28,8 +28,8 @@
 #include "align.h"
 #include "log.h"
 
-#include <qobject.h>
-#include <qapplication.h>
+#include <QtCore/QObject>
+#include <QtGui/QApplication>
 
 AlignCommand::AlignCommand()
 {
