@@ -34,20 +34,19 @@
 #include "3dmprefs.h"
 #include "mm3dport.h"
 
-#include "mq3compat.h"
-
 #include "pixmap/arrow.xpm"
 #include "pixmap/crosshairrow.xpm"
 
-#include <qfont.h>
-#include <qtimer.h>
+#include <QtCore/QTimer>
+#include <QtGui/QFont>
+#include <QtGui/QPixmap>
+#include <QtGui/QFocusEvent>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QWheelEvent>
 #include <math.h>
 #include <stdarg.h>
 
-#ifdef HAVE_QT4
-#include <QMouseEvent>
-#include <QWheelEvent>
-#endif // HAVE_QT4
 
 #define NEWVIEWPORT
 
@@ -84,8 +83,8 @@ static ScrollButtonT s_buttons[ ModelViewport::ScrollButtonMAX ] =
 
 static Matrix s_mat;
 
-ModelViewport::ModelViewport( QWidget * parent, const char * name )
-   : QGLWidget( parent, name ),
+ModelViewport::ModelViewport( QWidget * parent )
+   : QGLWidget( parent ),
      m_model( NULL ),
      m_operation( MO_None ),
      m_activeButton( Qt::NoButton ),
@@ -115,10 +114,12 @@ ModelViewport::ModelViewport( QWidget * parent, const char * name )
    m_arcballPoint[1] = 0.0;
    m_arcballPoint[2] = 0.0;
 
+   m_scrollTimer->setSingleShot( true );
+
    g_prefs.setDefault( "ui_grid_inc", 4.0 );
 
    g_prefs.setDefault( "ui_3dgrid_inc", 4.0 );
-   g_prefs.setDefault( "ui_3dgrid_count", 6.0 );
+   g_prefs.setDefault( "ui_3dgrid_count", 6 );
 
    g_prefs.setDefault( "ui_3dgrid_xy", 0 );
    g_prefs.setDefault( "ui_3dgrid_xz", 1 );
@@ -126,7 +127,7 @@ ModelViewport::ModelViewport( QWidget * parent, const char * name )
 
    setAutoBufferSwap( false );
 
-   setFocusPolicy( WheelFocus );
+   setFocusPolicy( Qt::WheelFocus );
    setMinimumSize( 220, 180 );
 
    double rot[3] = { 45 * PIOVER180, 0, 0 };
@@ -146,10 +147,10 @@ ModelViewport::ModelViewport( QWidget * parent, const char * name )
 
    glGenTextures( 2, m_scrollTextures );
 
-   img = arrow.convertToImage();
+   img = arrow.toImage();
    makeTextureFromImage( img, m_scrollTextures[0] );
 
-   img = cross.convertToImage();
+   img = cross.toImage();
    makeTextureFromImage( img, m_scrollTextures[1] );
 
    connect( m_scrollTimer, SIGNAL(timeout()), this, SLOT(scrollTimeout()));
@@ -759,9 +760,17 @@ void ModelViewport::drawGridLines()
             log_error( "Unhandled view direction: %d\n", m_viewDirection );
             break;
       }
-      QString text;
-      text.sprintf( "%g", unitWidth );
-      renderText( 2, this->height() - 12, text, QFont( "Fixed", 10 ) );
+
+      g_prefs.setDefault( "ui_render_text", 0 );
+      if ( g_prefs( "ui_render_text" ).intValue() != 0 )
+      {
+         // Broken with Qt4 + non-accelerated nVidia card (possibly other configs as
+         // well). That is why it is disabled by default. You can set ui_render_text
+         // to a non-zero value in your mm3drc file to enable text rendering.
+         QString text;
+         text.sprintf( "%g", unitWidth );
+         renderText( 2, this->height() - 12, text, QFont( "Sans", 10 ) );
+      }
    }
 }
 
@@ -1035,7 +1044,7 @@ void ModelViewport::updateBackground()
          if ( !m_texture )
          {
             QString str = tr("Could not load background %1").arg( m_backgroundFile.c_str() );
-            model_status( m_model, StatusError, STATUSTIME_LONG, "%s", (const char *) str.utf8() );
+            model_status( m_model, StatusError, STATUSTIME_LONG, "%s", (const char *) str.toUtf8() );
             m_texture = TextureManager::getInstance()->getDefaultTexture( m_backgroundFile.c_str() );
          }
          glBindTexture( GL_TEXTURE_2D, m_backgroundTexture );
@@ -1205,7 +1214,7 @@ void ModelViewport::wheelEvent( QWheelEvent * e )
 {
    if ( e->delta() > 0 )
    {
-      if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+      if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
       {
          rotateClockwise();
       }
@@ -1216,7 +1225,7 @@ void ModelViewport::wheelEvent( QWheelEvent * e )
    }
    else
    {
-      if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+      if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
       {
          rotateCounterClockwise();
       }
@@ -1373,7 +1382,7 @@ void ModelViewport::mousePressEvent( QMouseEvent * e )
       {
          m_overlayButton = (ScrollButtonE) b;
 
-         if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+         if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
          {
             m_operation = ( m_overlayButton == ScrollButtonPan )
                ? MO_Rotate : MO_RotateButton;
@@ -1394,28 +1403,32 @@ void ModelViewport::mousePressEvent( QMouseEvent * e )
                   rotateUp();
                else 
                   scrollUp();
-               m_scrollTimer->start( 300, true );
+               m_scrollTimer->setSingleShot( true );
+               m_scrollTimer->start( 300 );
                break;
             case ScrollButtonDown:
                if ( m_operation == MO_RotateButton )
                   rotateDown();
                else 
                   scrollDown();
-               m_scrollTimer->start( 300, true );
+               m_scrollTimer->setSingleShot( true );
+               m_scrollTimer->start( 300 );
                break;
             case ScrollButtonLeft:
                if ( m_operation == MO_RotateButton )
                   rotateLeft();
                else 
                   scrollLeft();
-               m_scrollTimer->start( 300, true );
+               m_scrollTimer->setSingleShot( true );
+               m_scrollTimer->start( 300 );
                break;
             case ScrollButtonRight:
                if ( m_operation == MO_RotateButton )
                   rotateRight();
                else 
                   scrollRight();
-               m_scrollTimer->start( 300, true );
+               m_scrollTimer->setSingleShot( true );
+               m_scrollTimer->start( 300 );
                break;
             default:
                break;
@@ -1425,13 +1438,13 @@ void ModelViewport::mousePressEvent( QMouseEvent * e )
 
    if ( m_operation == MO_None )
    {
-      if ( e->button() == MidButton ) 
+      if ( e->button() == Qt::MidButton ) 
       {
          m_operation = MO_Pan;
          m_scrollStartPosition = e->pos();
       }
-      else if ( (m_viewDirection == ViewPerspective && e->button() == LeftButton) 
-            || (e->state() & Qt::ControlButton) )
+      else if ( (m_viewDirection == ViewPerspective && e->button() == Qt::LeftButton) 
+            || (e->modifiers() & Qt::ControlModifier) )
       {
          m_operation = MO_Rotate;
          m_scrollStartPosition = e->pos();
@@ -1449,7 +1462,7 @@ void ModelViewport::mousePressEvent( QMouseEvent * e )
    /*
    if ( m_overlayButton == ScrollButtonMAX )
    {
-      if ( e->button() == MidButton || (m_viewDirection == ViewPerspective && e->button() == LeftButton) )
+      if ( e->button() == Qt::MidButton || (m_viewDirection == ViewPerspective && e->button() == Qt::LeftButton) )
       {
          m_scrollStartPosition = e->pos();
       }
@@ -1488,8 +1501,12 @@ void ModelViewport::mouseReleaseEvent( QMouseEvent * e )
 
             if ( m_model->getSelectedBoneJointCount() > 0 )
             {
-               m_model->setDrawJoints( 
-                     static_cast<Model::DrawJointModeE>( g_prefs( "ui_draw_joints" ).intValue() ) );
+               Model::DrawJointModeE mode = 
+                  static_cast<Model::DrawJointModeE>(
+                        g_prefs( "ui_draw_joints" ).intValue() );
+               if ( mode == Model::JOINTMODE_NONE )
+                  mode = Model::JOINTMODE_BONES;
+               m_model->setDrawJoints( mode );
                updateView();
             }
          }
@@ -1499,9 +1516,9 @@ void ModelViewport::mouseReleaseEvent( QMouseEvent * e )
          m_overlayButton = ScrollButtonMAX;
          m_scrollTimer->stop();
 
-         model_status( m_model, StatusNormal, STATUSTIME_SHORT, tr("Use the middle mouse button to drag/pan the viewport").utf8() );
+         model_status( m_model, StatusNormal, STATUSTIME_SHORT, tr("Use the middle mouse button to drag/pan the viewport").toUtf8() );
       }
-      m_activeButton = NoButton;
+      m_activeButton = Qt::NoButton;
       m_operation    = MO_None;
    }
 }
@@ -1631,9 +1648,9 @@ void ModelViewport::mouseMoveEvent( QMouseEvent * e )
    {
       if ( m_activeButton )
       {
-         if ( m_activeButton == MidButton || (m_viewDirection == ViewPerspective && m_activeButton == LeftButton) )
+         if ( m_activeButton == Qt::MidButton || (m_viewDirection == ViewPerspective && m_activeButton == Qt::LeftButton) )
          {
-            if ( m_viewDirection == ViewPerspective && (m_activeButton == LeftButton) )
+            if ( m_viewDirection == ViewPerspective && (m_activeButton == Qt::LeftButton) )
             {
                QPoint curPos = e->pos();
 
@@ -1794,10 +1811,10 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
    {
       switch ( e->key() )
       {
-         case Key_Equal:
-         case Key_Plus:
+         case Qt::Key_Equal:
+         case Qt::Key_Plus:
             {
-               if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+               if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
                {
                   rotateClockwise();
                }
@@ -1807,10 +1824,10 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
                }
             }
             break;
-         case Key_Minus:
-         case Key_Underscore:
+         case Qt::Key_Minus:
+         case Qt::Key_Underscore:
             {
-               if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+               if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
                {
                   rotateCounterClockwise();
                }
@@ -1820,7 +1837,7 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
                }
             }
             break;
-         case Key_QuoteLeft:
+         case Qt::Key_QuoteLeft:
             {
                int newDir = 0;
                switch ( m_viewDirection )
@@ -1844,7 +1861,7 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
                emit viewDirectionChanged( newDir );
             }
             break;
-         case Key_Backslash:
+         case Qt::Key_Backslash:
             {
                int newDir = 0;
                switch ( m_viewDirection )
@@ -1880,7 +1897,7 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
                emit viewDirectionChanged( newDir );
             }
             break;
-         case Key_0:
+         case Qt::Key_0:
             m_centerX = 0.0;
             m_centerY = 0.0;
             m_arcballPoint[0] = 0.0;
@@ -1889,48 +1906,44 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
             makeCurrent();
             adjustViewport();
             break;
-         case Key_Up:
-            if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+         case Qt::Key_Up:
+            if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
                rotateUp();
             else
                scrollUp();
             break;
-         case Key_Down:
-            if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+         case Qt::Key_Down:
+            if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
                rotateDown();
             else
                scrollDown();
             break;
-         case Key_Left:
-            if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+         case Qt::Key_Left:
+            if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
                rotateLeft();
             else
                scrollLeft();
             break;
-         case Key_Right:
-            if ( (e->state() & Qt::ControlButton) == Qt::ControlButton )
+         case Qt::Key_Right:
+            if ( (e->modifiers() & Qt::ControlModifier) == Qt::ControlModifier )
                rotateRight();
             else
                scrollRight();
             break;
-         case Key_1:
-         case Key_2:
-         case Key_3:
-         case Key_4:
-         case Key_5:
-         case Key_6:
-         case Key_7:
-         case Key_8:
-         case Key_9:
-#ifdef HAVE_QT4
-            if ( e->modifiers() & ControlModifier )
-#else
-            if ( e->state() & ControlButton )
-#endif // HAVE_QT4
+         case Qt::Key_1:
+         case Qt::Key_2:
+         case Qt::Key_3:
+         case Qt::Key_4:
+         case Qt::Key_5:
+         case Qt::Key_6:
+         case Qt::Key_7:
+         case Qt::Key_8:
+         case Qt::Key_9:
+            if ( e->modifiers() & Qt::ControlModifier )
             {
-               log_debug( "set viewport %d\n", e->key() - Key_1 );
+               log_debug( "set viewport %d\n", e->key() - Qt::Key_1 );
 
-               int slot = ( (int) e->key() - (int) Key_1 );
+               int slot = ( (int) e->key() - (int) Qt::Key_1 );
                ViewStateT viewState;
 
                viewState.direction = m_viewDirection;
@@ -1946,8 +1959,8 @@ void ModelViewport::keyPressEvent( QKeyEvent * e )
             }
             else
             {
-               log_debug( "viewport recall %d\n", e->key() - Key_1 );
-               int slot = ( (int) e->key() - (int) Key_1 );
+               log_debug( "viewport recall %d\n", e->key() - Qt::Key_1 );
+               int slot = ( (int) e->key() - (int) Qt::Key_1 );
                emit viewportRecallState( slot );
                return;
             }
@@ -2264,6 +2277,7 @@ void ModelViewport::scrollTimeout()
       }
    }
 
+   m_scrollTimer->setSingleShot( false );
    m_scrollTimer->start( 100 );
 }
 
@@ -2691,42 +2705,30 @@ int ModelViewport::constructButtonState( QMouseEvent * e )
    //switch ( e->button() )
    switch ( m_activeButton )
    {
-      case LeftButton:
+      case Qt::LeftButton:
          button = ::Tool::BS_Left;
          break;
-      case MidButton:
+      case Qt::MidButton:
          button = ::Tool::BS_Middle;
          break;
-      case RightButton:
+      case Qt::RightButton:
          button = ::Tool::BS_Right;
          break;
       default:
          break;
    }
 
-#ifdef HAVE_QT4
-   if ( e->modifiers() & ShiftButton )
-#else
-   if ( e->state() & ShiftButton )
-#endif // HAVE_QT4
+   if ( e->modifiers() & Qt::ShiftModifier )
    {
       button |= ::Tool::BS_Shift;
    }
 
-#ifdef HAVE_QT4
-   if ( e->modifiers() & AltButton )
-#else
-   if ( e->state() & AltButton )
-#endif // HAVE_QT4
+   if ( e->modifiers() & Qt::AltModifier )
    {
       button |= ::Tool::BS_Alt;
    }
 
-#ifdef HAVE_QT4
-   if ( e->modifiers() & ControlButton )
-#else
-   if ( e->state() & ControlButton )
-#endif // HAVE_QT4
+   if ( e->modifiers() & Qt::ControlModifier )
    {
       button |= ::Tool::BS_Ctrl;
    }
@@ -2986,25 +2988,25 @@ void ModelViewport::checkGlErrors()
       switch ( error )
       {
          case GL_INVALID_VALUE:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Invalid Value").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Invalid Value").toUtf8() );
             break;
          case GL_INVALID_ENUM:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Invalid Enum").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Invalid Enum").toUtf8() );
             break;
          case GL_INVALID_OPERATION:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Invalid Operation").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Invalid Operation").toUtf8() );
             break;
          case GL_STACK_OVERFLOW:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Stack Overflow").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Stack Overflow").toUtf8() );
             break;
          case GL_STACK_UNDERFLOW:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Stack Underflow").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Stack Underflow").toUtf8() );
             break;
          case GL_OUT_OF_MEMORY:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Out Of Memory").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Out Of Memory").toUtf8() );
             break;
          default:
-            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Unknown").utf8() );
+            model_status( m_model, StatusNormal, STATUSTIME_NONE, tr("OpenGL error = Unknown").toUtf8() );
             break;
       }
    }
