@@ -25,7 +25,7 @@
 
 #include "version.h"
 
-#include <qtextbrowser.h>
+#include <QtGui/QTextBrowser>
 
 #define ABOUT_TEXT "<html><head><title>Misfit Model 3D - About</title></head> " \
    "<body><center><br>" \
@@ -36,11 +36,17 @@
    "<font color=\"blue\">kevin&nbsp;at</font>&nbsp;the&nbsp;<font color=\"blue\">misfitcode.com</font> domain<br>" \
    "</center></body></html>" 
 
-AboutWin::AboutWin( QWidget * parent, const char * name )
-   : TextWinBase( parent, name, false, Qt::WDestructiveClose )
+AboutWin::AboutWin( QWidget * parent )
+   : QDialog( parent )
 {
-   setCaption( tr( "Misfit Model 3D - About") );
-   m_text->setText( QString( ABOUT_TEXT ) );
+   setAttribute( Qt::WA_DeleteOnClose );
+   setupUi( this );
+
+   setModal( false );
+   setWindowTitle( tr( "Misfit Model 3D - About") );
+
+   resize( 350, 300 );
+   m_text->setHtml( QString( ABOUT_TEXT ) );
 }
 
 AboutWin::~AboutWin()
