@@ -24,21 +24,21 @@
 #ifndef __BACKGROUNDSELECT_H
 #define __BACKGROUNDSELECT_H
 
-#include <qpixmap.h>
 #include "backgroundselect.base.h"
 
-#include "mq3macro.h"
+#include <QtGui/QWidget>
 
 class Model;
 
-class BackgroundSelect : public BackgroundSelectBase
+class BackgroundSelect : public QWidget, public Ui::BackgroundSelectBase
 {
    Q_OBJECT
 
    public:
-      BackgroundSelect( Model * mode, unsigned index, QWidget * parent = NULL, const char * name = "" );
+      BackgroundSelect( Model * mode, unsigned index, QWidget * parent = NULL );
       virtual ~BackgroundSelect();
 
+   public slots:
       void setFilename( const char * filename );
       void noneEvent();
       void selectFileEvent();
