@@ -28,12 +28,11 @@
 #include "modelfilter.h"
 #include "cal3dfilter.h"
 
-#include "mq3macro.h"
+#include <QtGui/QDialog>
 
-class QAccel;
 class Model;
 
-class Cal3dPrompt : public Cal3dPromptBase
+class Cal3dPrompt : public QDialog, public Ui::Cal3dPromptBase
 {
    Q_OBJECT
 
@@ -45,11 +44,9 @@ class Cal3dPrompt : public Cal3dPromptBase
       void getOptions( Cal3dFilter::Cal3dOptions * o );
 
    public slots:
-      void helpNowEvent( int id );
+      void helpNowEvent();
 
    protected:
-
-      QAccel * m_accel;
 };
 
 bool cal3dprompt_show( Model * model, ModelFilter::Options * o );
