@@ -25,16 +25,21 @@
 
 #include "sysconf.h"
 
-#include <qtextbrowser.h>
+#include <QtGui/QTextBrowser>
 
-LicenseWin::LicenseWin( QWidget * parent, const char * name )
-   : TextWinBase( parent, name, false, Qt::WDestructiveClose )
+LicenseWin::LicenseWin( QWidget * parent )
+   : QDialog( parent )
 {
+   setAttribute( Qt::WA_DeleteOnClose );
+   setModal( false );
+   setupUi( this );
+
    resize( 600, 400 );
-   setCaption( tr("GNU General Public License") );
+   setWindowTitle( tr("GNU General Public License") );
 #ifdef WIN32
    QString source = 
-        QString( getDocDirectory().c_str() )
+        QString( "file:///")
+      + QString( getDocDirectory().c_str() )
       + QString( "/olh_license.html" );
 #else
    QString source = QString( "file://" ) 
