@@ -26,21 +26,21 @@
 
 #include "pointwin.base.h"
 
-#include "mq3macro.h"
+#include <QtGui/QDialog>
 
 class Model;
-class QAccel;
+class Q3Accel;
 
-class PointWin : public PointWinBase
+class PointWin : public QDialog, public Ui::PointWinBase
 {
    Q_OBJECT
 
    public:
-      PointWin( Model * model, QWidget * parent = NULL, const char * name = "" );
+      PointWin( Model * model, QWidget * parent = NULL );
       virtual ~PointWin();
 
    public slots:
-      void helpNowEvent( int );
+      void helpNowEvent();
 
       void deleteClicked();
       void renameClicked();
@@ -52,7 +52,7 @@ class PointWin : public PointWinBase
       void reject();
 
    protected:
-      QAccel * m_accel;
+      Q3Accel * m_accel;
       Model  * m_model;
 };
 
