@@ -31,23 +31,23 @@
 #include "decalmgr.h"
 #include "helpwin.h"
 
-#include "mq3compat.h"
-
 #include "keycfg.h"
 #include "3dmprefs.h"
 
-#include <qpushbutton.h>
-#include <qbuttongroup.h>
-#include <qradiobutton.h>
-#include <qcheckbox.h>
-#include <qmessagebox.h>
-#include <qslider.h>
-#include <qlineedit.h>
+#include <QtGui/QPushButton>
+#include <QtGui/QButtonGroup>
+#include <QtGui/QRadioButton>
+#include <QtGui/QCheckBox>
+#include <QtGui/QMessageBox>
+#include <QtGui/QSlider>
+#include <QtGui/QLineEdit>
+#include <QtGui/QPixmap>
+#include <QtGui/QShortcut>
+
 #include <math.h>
 
-TextureCoord::TextureCoord( Model * model, QWidget * parent, const char * name )
-   : TextureCoordBase( parent, name ),
-     m_accel( new QAccel(this) ),
+TextureCoord::TextureCoord( Model * model, QWidget * parent )
+   : QDialog( parent ),
      m_undoCount( 0 ),
      m_redoCount( 0 ),
      m_inUndo( false ),
@@ -55,20 +55,29 @@ TextureCoord::TextureCoord( Model * model, QWidget * parent, const char * name )
      m_currentDirection( 0 ),
      m_currentMapScheme( 2 ) // if you change this, change the setChecked line below also
 {
-   // FIXME handle undo of select/unselect?
+   setupUi( this );
+
+   // TODO handle undo of select/unselect?
    // Can't do this until after constructor is done because of observer interface
    //setModel( m_model );
    m_textureFrame->setModel( model );
 
-   m_accel->insertItem( Key_F1, HELP_ID );
-   connect( m_accel, SIGNAL(activated(int)), this, SLOT(accelEvent(int)) );
+   QShortcut * help = new QShortcut( QKeySequence( tr("F1", "Help Shortcut")), this );
+   connect( help, SIGNAL(activated()), this, SLOT(helpNowEvent()) );
 
-   m_accel->insertItem( Qt::CTRL + Qt::Key_Z, UNDO_ID );
-   m_accel->insertItem( Qt::CTRL + Qt::Key_Y, REDO_ID );
-   m_accel->insertItem( g_keyConfig.getKey( "tool_select_vertices" ), TOOL_SELECT_ID );
-   m_accel->insertItem( g_keyConfig.getKey( "tool_move" ), TOOL_MOVE_ID );
-   m_accel->insertItem( g_keyConfig.getKey( "tool_rotate" ), TOOL_ROTATE_ID );
-   m_accel->insertItem( g_keyConfig.getKey( "tool_scale" ), TOOL_SCALE_ID );
+   QShortcut * undo = new QShortcut( QKeySequence( tr("CTRL+Z", "Undo shortcut")), this );
+   connect( undo, SIGNAL(activated()), this, SLOT(undoEvent()) );
+   QShortcut * redo = new QShortcut( QKeySequence( tr("CTRL+Y", "Redo shortcut")), this );
+   connect( redo, SIGNAL(activated()), this, SLOT(redoEvent()) );
+
+   QShortcut * select = new QShortcut( g_keyConfig.getKey( "tool_select_vertices" ), this );
+   connect( select, SIGNAL(activated()), this, SLOT(toolSelectEvent()) );
+   QShortcut * move = new QShortcut( g_keyConfig.getKey( "tool_move" ), this );
+   connect( move, SIGNAL(activated()), this, SLOT(toolMoveEvent()) );
+   QShortcut * rotate = new QShortcut( g_keyConfig.getKey( "tool_rotate" ), this );
+   connect( rotate, SIGNAL(activated()), this, SLOT(toolRotateEvent()) );
+   QShortcut * scale = new QShortcut( g_keyConfig.getKey( "tool_scale" ), this );
+   connect( scale, SIGNAL(activated()), this, SLOT(toolScaleEvent()) );
 
    m_textureWidget = m_textureFrame->getTextureWidget();
    m_textureWidget->setInteractive( true );
@@ -90,7 +99,7 @@ TextureCoord::TextureCoord( Model * model, QWidget * parent, const char * name )
    connect( m_textureWidget, SIGNAL(zoomLevelChanged(QString)), this, SLOT(zoomLevelChangedEvent(QString)) );
 
    m_selectButton->setChecked( true );
-   mouseToolChangedEvent( 0 );
+   m_textureWidget->setMouseOperation( TextureWidget::MouseSelect );
 
    m_groupButton->setChecked( true );  // if you change this, change m_currentMapScheme also
 }
@@ -107,6 +116,7 @@ void TextureCoord::undoEvent()
       m_model->undo();
       m_undoCount--;
       m_inUndo = false;
+      m_textureWidget->updateGL();
    }
 }
 
@@ -118,6 +128,7 @@ void TextureCoord::redoEvent()
       m_model->redo();
       m_undoCount++;
       m_inUndo = false;
+      m_textureWidget->updateGL();
    }
 }
 
@@ -140,7 +151,7 @@ void TextureCoord::show()
       // If we are visible, setModel already did this
       initWindow();
    }
-   TextureCoordBase::show();
+   QDialog::show();
 }
 
 void TextureCoord::initWindow()
@@ -195,110 +206,57 @@ void TextureCoord::setModel( Model * model )
    }
 }
 
-void TextureCoord::accelEvent( int id )
+void TextureCoord::helpNowEvent()
 {
-   switch ( id )
-   {
-      case HELP_ID:
-         {
-            HelpWin * win = new HelpWin( "olh_texturecoordwin.html", true );
-            win->show();
-         }
-         break;
+   HelpWin * win = new HelpWin( "olh_texturecoordwin.html", true );
+   win->show();
+}
 
-      case UNDO_ID:
-         undoEvent();
-         break;
-      case REDO_ID:
-         redoEvent();
-         break;
-
-      case TOOL_SELECT_ID:
-         m_selectButton->setChecked( true );
-         m_textureWidget->setMouseOperation( TextureWidget::MouseSelect );
-         break;
-
-      case TOOL_MOVE_ID:
-         m_moveButton->setChecked( true );
-         m_textureWidget->setMouseOperation( TextureWidget::MouseMove );
-         break;
-
-      case TOOL_ROTATE_ID:
-         m_rotateButton->setChecked( true );
-         m_textureWidget->setMouseOperation( TextureWidget::MouseRotate );
-         break;
-
-      case TOOL_SCALE_ID:
-         m_scaleButton->setChecked( true );
-         m_textureWidget->setMouseOperation( TextureWidget::MouseScale );
-         break;
-
-      default:
-         break;
-   }
+void TextureCoord::toolSelectEvent()
+{
+   m_selectButton->setChecked( true );
+   m_textureWidget->setMouseOperation( TextureWidget::MouseSelect );
    m_textureWidget->updateGL();
 }
 
-void TextureCoord::mapSchemeChangedEvent( int scheme )
+void TextureCoord::toolMoveEvent()
 {
-   if ( scheme == m_currentMapScheme )
+   m_moveButton->setChecked( true );
+   m_textureWidget->setMouseOperation( TextureWidget::MouseMove );
+   m_textureWidget->updateGL();
+}
+
+void TextureCoord::toolRotateEvent()
+{
+   m_rotateButton->setChecked( true );
+   m_textureWidget->setMouseOperation( TextureWidget::MouseRotate );
+   m_textureWidget->updateGL();
+}
+
+void TextureCoord::toolScaleEvent()
+{
+   m_scaleButton->setChecked( true );
+   m_textureWidget->setMouseOperation( TextureWidget::MouseScale );
+   m_textureWidget->updateGL();
+}
+
+void TextureCoord::mapGroupEvent()
+{
+   if ( m_currentMapScheme == MapSchemeGroup )
    {
       return;
    }
 
-   switch ( scheme )
+   MapDirection dir;
+   dir.setMapDirection( getDefaultDirection() );
+   if ( dir.exec() )
    {
-      case MapSchemeTriangle:
-         m_currentMapScheme = scheme;
-         mapTriangle();
-         break;
-      case MapSchemeQuad:
-         m_currentMapScheme = scheme;
-         mapQuad();
-         break;
-      case MapSchemeGroup:
-         {
-            MapDirection dir;
-            dir.setMapDirection( getDefaultDirection() );
-            if ( dir.exec() )
-            {
-               m_currentMapScheme = scheme;
-               mapGroup( dir.getMapDirection() );
-            }
-            else
-            {
-               cancelMapChange();
-               return;
-            }
-         }
-         break;
-      default:
-         log_error( "unknown map scheme: %d\n", scheme );
-         return; // Error, don't set m_currentMapScheme
-         break;
+      mapGroup( dir.getMapDirection() );
    }
-
-   m_textureWidget->updateGL();
-}
-
-void TextureCoord::mouseToolChangedEvent( int tool )
-{
-   switch ( tool )
+   else
    {
-      case 0:
-         m_textureWidget->setMouseOperation( TextureWidget::MouseSelect );
-         break;
-      case 1:
-         m_textureWidget->setMouseOperation( TextureWidget::MouseMove );
-         break;
-      case 2:
-         m_textureWidget->setMouseOperation( TextureWidget::MouseRotate );
-         break;
-      case 3:
-         m_textureWidget->setMouseOperation( TextureWidget::MouseScale );
-         break;
-      default:
-         break;
+      cancelMapChange();
+      return;
    }
    m_textureWidget->updateGL();
 }
@@ -339,7 +297,7 @@ void TextureCoord::resetClickedEvent()
 
 void TextureCoord::updateDoneEvent()
 {
-   operationComplete( tr("Move texture coordinates").utf8() );
+   operationComplete( tr("Move texture coordinates").toUtf8() );
 }
 
 void TextureCoord::updateTextureCoordsEvent()
@@ -421,7 +379,7 @@ void TextureCoord::zoomOut()
 
 void TextureCoord::zoomChangeEvent()
 {
-   double zoom = atof( m_zoomInput->text().latin1() );
+   double zoom = atof( m_zoomInput->text().toLatin1() );
    if ( zoom < 0.00001 )
    {
       zoom = 1;
@@ -436,6 +394,13 @@ void TextureCoord::zoomLevelChangedEvent( QString zoomStr )
 
 void TextureCoord::mapTriangle()
 {
+   if ( MapSchemeTriangle == m_currentMapScheme )
+   {
+      return;
+   }
+
+   m_currentMapScheme = MapSchemeTriangle;
+
    m_textureWidget->clearCoordinates();
    clearTriangles();
 
@@ -449,10 +414,18 @@ void TextureCoord::mapTriangle()
    m_triangles.push_back( m_textureWidget->addTriangle( v1, v2, v3 ) );
    updateTextureCoordsEvent();
    updateDoneEvent();
+   m_textureWidget->updateGL();
 }
 
 void TextureCoord::mapQuad()
 {
+   if ( MapSchemeQuad == m_currentMapScheme )
+   {
+      return;
+   }
+
+   m_currentMapScheme = MapSchemeQuad;
+
    m_textureWidget->clearCoordinates();
    clearTriangles();
 
@@ -468,12 +441,21 @@ void TextureCoord::mapQuad()
    m_triangles.push_back( m_textureWidget->addTriangle( v1, v3, v4 ) );
    updateTextureCoordsEvent();
    updateDoneEvent();
+   m_textureWidget->updateGL();
 }
 
 void TextureCoord::mapGroup( int direction )
 {
    log_debug( "mapGroup( %d )\n", direction );
 
+   if ( MapSchemeGroup == m_currentMapScheme )
+   {
+      log_debug( "  ignored\n" );
+      return;
+   }
+
+   m_currentMapScheme = MapSchemeGroup;
+
    double range = 1.0; //m_textureWidget->getMaxViewCoord() - m_textureWidget->getMinViewCoord();
    double xOff  = 0.0; //m_textureWidget->getMinViewCoord();
    double yOff  = xOff;
@@ -588,6 +570,7 @@ void TextureCoord::mapGroup( int direction )
 
    updateTextureCoordsEvent();
    updateDoneEvent();
+   m_textureWidget->updateGL();
 }
 
 void TextureCoord::clearTriangles()
