@@ -23,27 +23,23 @@
 #ifndef __VIEWPORTSETTINGS_H
 #define __VIEWPORTSETTINGS_H
 
-#include "mq3macro.h"
-#include "mq3compat.h"
-
 #include "viewportsettings.base.h"
 
-class QAccel;
+#include <QtGui/QDialog>
 
-class ViewportSettings : public ViewportSettingsBase
+class ViewportSettings : public QDialog, public Ui::ViewportSettingsBase
 {
    Q_OBJECT
 
    public:
-      ViewportSettings( QWidget * parent = NULL, const char * name = "" );
+      ViewportSettings( QWidget * parent = NULL );
       virtual ~ViewportSettings();
 
    public slots:
-      void helpNowEvent( int );
+      void helpNowEvent();
       void accept();
 
    private:
-      QAccel * m_accel;
 };
 
 #endif // __VIEWPORTSETTINGS_H
