@@ -987,7 +987,7 @@ bool CobFilter::readAMaterialChunk( const CobFilter::ChunkHeaderT & header, CobF
        std::string newPath = m_modelPath + "/";
        log_debug( "texture filename is '%s'\n", file.c_str() );
 
-       material.texFile = fixFileCase( newPath.c_str(), file.c_str() );
+       material.texFile = newPath + fixFileCase( newPath.c_str(), file.c_str() );
 
        skipAString( "offset" );
        material.uOffset = readAFloat();
@@ -1271,10 +1271,10 @@ bool CobFilter::readBMaterialChunk( const CobFilter::ChunkHeaderT & header, CobF
 
        std::string file = getFileNameFromPath( path.c_str() );
 
-       file = m_modelPath + std::string("/") + file;
        log_debug( "texture filename is '%s'\n", file.c_str() );
 
-       material.texFile = file;
+       std::string newPath = m_modelPath + std::string("/");
+       material.texFile = newPath + fixFileCase( newPath.c_str(), file.c_str() );
 
        material.uOffset = readBFloat();
        material.vOffset = readBFloat();
@@ -1498,7 +1498,7 @@ void CobFilter::writeBTriangleGroup( const std::list<int> & triList, const std::
     writeBLong( uvcount );
 
     // Write uv data
-    // FIXME deal with out of range UVs (related to UV Offset/Repeat on Material?)
+    // TODO deal with out of range UVs (related to UV Offset/Repeat on Material?)
     for ( size_t uv = 0; uv < uvcount; uv++ )
     {
         writeBFloat( uvList[uv].u );
@@ -1671,7 +1671,7 @@ void CobFilter::writeBMaterial( int32_t parentId, size_t materialNumber, int gro
           texFile = getFileNameFromPath( texFile.c_str() );
           writeBString( texFile );
 
-          // FIXME make sure these are handled correctly
+          // TODO make sure these are handled correctly
           writeBFloat( 0.0 ); // U Offset
           writeBFloat( 0.0 ); // V Offset
           writeBFloat( 1.0 ); // U Repeat
