@@ -74,6 +74,7 @@ bool FilterManager::registerFilter( ModelFilter * filter )
 {
    if ( filter )
    {
+      filter->setFactory( &m_factory );
       m_filters.push_back( filter );
       return true;
    }
@@ -85,25 +86,36 @@ bool FilterManager::registerFilter( ModelFilter * filter )
 
 Model::ModelErrorE FilterManager::readFile( Model * model, const char * filename )
 {
+   Model::ModelErrorE rval = Model::ERROR_UNKNOWN_TYPE;
    FilterList::iterator it;
    for ( it = m_filters.begin(); it != m_filters.end(); it++ )
    {
       ModelFilter * filter = *it;
 
-      if ( filter && filter->isSupported( filename ) && filter->canRead() )
+      if ( filter && filter->isSupported( filename ) )
       {
-         model->forceAddOrDelete( true );
-         Model::ModelErrorE rval = filter->readFile( model, filename );
-         model->forceAddOrDelete( false );
-#ifdef MM3D_EDIT
-         model->setUndoEnabled( true );
-#endif // MM3D_EDIT
-         return rval;
-      }
+         if ( filter->canRead() )
+         {
+            model->setUndoEnabled( false );
+            model->forceAddOrDelete( true );
 
+            rval = filter->readFile( model, filename );
+
+            model->forceAddOrDelete( false );
+            model->setUndoEnabled( true );
+            model->clearUndo();
+            m_factory.closeAll();
+
+            return rval;
+         }
+         else
+         {
+            rval = Model::ERROR_UNSUPPORTED_OPERATION;
+         }
+      }
    }
 
-   return Model::ERROR_UNKNOWN_TYPE;
+   return rval;
 }
 
 Model::ModelErrorE FilterManager::writeFile( Model * model, const char * filename, bool exportModel, FilterManager::WriteOptionsE wo )
@@ -153,6 +165,7 @@ Model::ModelErrorE FilterManager::writeFile( Model * model, const char * filenam
             if ( doWrite )
             {
                err = filter->writeFile( model, filename, o );
+               m_factory.closeAll();
             }
             else
             {
