@@ -1,6 +1,6 @@
 /*  Misfit Model 3D
  * 
- *  Copyright (c) 2004-2005 Kevin Worcester
+ *  Copyright (c) 2004-2007 Kevin Worcester
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,9 +52,9 @@ LwoFilter::~LwoFilter()
 {
 }
 
-Model::ModelError LwoFilter::readFile( Model * model, const char * const filename )
+Model::ModelErrorE LwoFilter::readFile( Model * model, const char * const filename )
 {
-   Model::ModelError err = Model::ERROR_NONE;
+   Model::ModelErrorE err = Model::ERROR_NONE;
 
    m_fp = fopen( filename, "rb" );
 
@@ -292,7 +292,7 @@ Model::ModelError LwoFilter::readFile( Model * model, const char * const filenam
          unsigned count = m_discVertexMaps.size();
          for ( unsigned t = 0; t < count; t++ )
          {
-            DiscVertexMapData * dvmd = &m_discVertexMaps[t];
+            DiscVertexMapDataT * dvmd = &m_discVertexMaps[t];
             unsigned polyBase = m_polyMaps[ dvmd->polyIndex ].polyIndex;
             unsigned count    = m_polyMaps[ dvmd->polyIndex ].polyCount;
             // TODO confirm vertex is triangle vertex index, not model vertex index
@@ -346,7 +346,7 @@ Model::ModelError LwoFilter::readFile( Model * model, const char * const filenam
    return err;
 }
 
-Model::ModelError LwoFilter::writeFile( Model * model, const char * const filename, ModelFilter::Options * o  )
+Model::ModelErrorE LwoFilter::writeFile( Model * model, const char * const filename, ModelFilter::Options * o  )
 {
    return Model::ERROR_UNSUPPORTED_OPERATION;
 }
@@ -365,6 +365,13 @@ bool LwoFilter::canWrite( const char * filename )
    return false;
 }
 
+bool LwoFilter::canExport( const char * filename )
+{
+   log_debug( "canExport( %s )\n", filename );
+   log_debug( "  false\n" );
+   return false;
+}
+
 bool LwoFilter::isSupported( const char * filename )
 {
    log_debug( "isSupported( %s )\n", filename );
@@ -468,7 +475,7 @@ bool LwoFilter::readPolygonChunk( size_t chunkLen )
          remaining -= 2;
       }
 
-      PolyMap pm;
+      PolyMapT pm;
       pm.polyIndex = m_model->getTriangleCount();
       pm.polyCount = (vcount >= 3) ? vcount - 2 : 0;
       m_polyMaps.push_back( pm );
@@ -643,7 +650,7 @@ bool LwoFilter::readSurfaceDefinitionChunk( size_t chunkLen )
       }
       else if ( m_isLWO2 && strncmp( id, "BLOK", 4 ) == 0 )
       {
-         SurfaceBlockData sbd;
+         SurfaceBlockDataT sbd;
          readSurfaceBlockChunk( subChunkLen, sbd );
          remaining -= subChunkLen;
 
@@ -704,7 +711,7 @@ bool LwoFilter::readSurfaceDefinitionChunk( size_t chunkLen )
    return true;
 }
 
-bool LwoFilter::readSurfaceBlockChunk( size_t chunkLen, SurfaceBlockData & sbd )
+bool LwoFilter::readSurfaceBlockChunk( size_t chunkLen, SurfaceBlockDataT & sbd )
 {
    int remaining = chunkLen;
 
@@ -782,7 +789,7 @@ bool LwoFilter::readVertexMapChunk( size_t chunkLen )
             remaining -= 4;
          }
 
-         VertexMapData vmd;
+         VertexMapDataT vmd;
          vmd.vertex = vert;
          vmd.s = val[0];
          vmd.t = val[1];
@@ -858,7 +865,7 @@ bool LwoFilter::readClipChunk( size_t chunkLen )
    unsigned clipIndex = readU4();
    remaining -= 4;
 
-   ClipData cd;
+   ClipDataT cd;
    cd.filename = "";
 
    while ( m_clips.size() < clipIndex )
