@@ -72,7 +72,7 @@ bool Model::setPointBoneJoint( unsigned point, int joint )
    return false;
 }
 
-list<int> Model::getBoneJointVertices( const int & joint )
+list<int> Model::getBoneJointVertices( int joint ) const
 {
    list<int> rval;
 
@@ -369,7 +369,7 @@ bool Model::removeAllPointInfluences( unsigned point )
    return false;
 }
 
-bool Model::getPositionInfluences( const Position & pos, InfluenceList & l )
+bool Model::getPositionInfluences( const Position & pos, InfluenceList & l ) const
 {
    switch ( pos.type )
    {
@@ -386,7 +386,7 @@ bool Model::getPositionInfluences( const Position & pos, InfluenceList & l )
    return false;
 }
 
-bool Model::getVertexInfluences( unsigned vertex, InfluenceList & l )
+bool Model::getVertexInfluences( unsigned vertex, InfluenceList & l ) const
 {
    if ( vertex < m_vertices.size() )
    {
@@ -397,7 +397,7 @@ bool Model::getVertexInfluences( unsigned vertex, InfluenceList & l )
    return false;
 }
 
-bool Model::getPointInfluences( unsigned point, InfluenceList & l )
+bool Model::getPointInfluences( unsigned point, InfluenceList & l ) const
 {
    if ( point < m_points.size() )
    {
@@ -408,69 +408,42 @@ bool Model::getPointInfluences( unsigned point, InfluenceList & l )
    return false;
 }
 
-int Model::getPrimaryPositionInfluence( const Position & pos )
+int Model::getPrimaryPositionInfluence( const Position & pos ) const
 {
-   switch ( pos.type )
+   InfluenceList l;
+   getPositionInfluences( pos, l );
+
+   Model::InfluenceT inf;
+   inf.m_boneId = -1;
+   inf.m_weight = -1.0;
+
+   InfluenceList::iterator it;
+   for ( it = l.begin(); it != l.end(); it++ )
    {
-      case PT_Vertex:
-         return getPrimaryVertexInfluence( pos.index );
-
-      case PT_Point:
-         return getPrimaryPointInfluence( pos.index );
-
-      default:
-         break;
+      if ( inf < *it )
+      {
+         inf = *it;
+      }
    }
-
-   return -1;
+   return inf.m_boneId;
 }
 
-int Model::getPrimaryVertexInfluence( unsigned vertex )
+int Model::getPrimaryVertexInfluence( unsigned vertex ) const
 {
-   if ( vertex < m_vertices.size() )
-   {
-      InfluenceList & l = m_vertices[ vertex ]->m_influences;
+   Position pos;
+   pos.type = PT_Vertex;
+   pos.index = vertex;
 
-      int joint = -1;
-      double weight = 0.0;
-
-      InfluenceList::iterator it;
-      for ( it = l.begin(); it != l.end(); it++ )
-      {
-         if ( (*it).m_weight > weight )
-         {
-            joint  = (*it).m_boneId;
-            weight = (*it).m_weight;
-         }
-      }
-      return joint;
-   }
-
-   return -1;
+   return getPrimaryPositionInfluence( pos );
 }
 
-int Model::getPrimaryPointInfluence( unsigned point )
+int Model::getPrimaryPointInfluence( unsigned point ) const
 {
-   if ( point < m_points.size() )
-   {
-      InfluenceList & l = m_points[ point ]->m_influences;
+   Position pos;
+   pos.type = PT_Point;
+   pos.index = point;
 
-      int joint = -1;
-      double weight = 0.0;
-
-      InfluenceList::iterator it;
-      for ( it = l.begin(); it != l.end(); it++ )
-      {
-         if ( (*it).m_weight > weight )
-         {
-            joint  = (*it).m_boneId;
-            weight = (*it).m_weight;
-         }
-      }
-      return joint;
-   }
-
-   return -1;
+   return getPrimaryPositionInfluence( pos );
 }
 
 bool Model::setPositionInfluenceType( const Position & pos, unsigned int joint, InfluenceTypeE type )
@@ -645,7 +618,7 @@ bool Model::setPointInfluenceWeight( unsigned point, unsigned int joint, double
    return false;
 }
 
-double Model::calculatePositionInfluenceWeight( const Position & pos, unsigned joint )
+double Model::calculatePositionInfluenceWeight( const Position & pos, unsigned joint ) const
 {
    switch ( pos.type )
    {
@@ -661,21 +634,21 @@ double Model::calculatePositionInfluenceWeight( const Position & pos, unsigned j
    return 0.0;
 }
 
-double Model::calculateVertexInfluenceWeight( unsigned vertex, unsigned joint )
+double Model::calculateVertexInfluenceWeight( unsigned vertex, unsigned joint ) const
 {
    double coord[3] = { 0, 0, 0 };
    getVertexCoords( vertex, coord );
    return calculateCoordInfluenceWeight( coord, joint );
 }
 
-double Model::calculatePointInfluenceWeight( unsigned point, unsigned joint )
+double Model::calculatePointInfluenceWeight( unsigned point, unsigned joint ) const
 {
    double coord[3] = { 0, 0, 0 };
    getPointCoords( point, coord );
    return calculateCoordInfluenceWeight( coord, joint );
 }
 
-double Model::calculateCoordInfluenceWeight( double * coord, unsigned joint )
+double Model::calculateCoordInfluenceWeight( const double * coord, unsigned joint ) const
 {
    if ( joint >= m_joints.size() )
    {
@@ -902,7 +875,7 @@ bool Model::autoSetCoordInfluences( double * coord, double sensitivity, bool sel
 }
 
 
-void Model::calculateRemainderWeight( Model::InfluenceList & list )
+void Model::calculateRemainderWeight( Model::InfluenceList & list ) const
 {
    int    remainders = 0;
    double remaining = 1.0;
