@@ -27,17 +27,6 @@
 
 #include "modelundo.h"
 
-// FIXME I really need to put this glmath
-static void _calculateNormal( double * normal,
-      double * a, double * b, double * c )
-{
-   normal[0] = a[1] * (b[2] - c[2]) + b[1] * (c[2] - a[2]) + c[1] * (a[2] - b[2]);
-   normal[1] = a[2] * (b[0] - c[0]) + b[2] * (c[0] - a[0]) + c[2] * (a[0] - b[0]);
-   normal[2] = a[0] * (b[1] - c[1]) + b[0] * (c[1] - a[1]) + c[0] * (a[1] - b[1]);
-
-   normalize3( normal );
-}
-
 void Model::setSelectionMode( Model::SelectionModeE m )
 {
    if ( m != m_selectionMode )
@@ -361,7 +350,7 @@ bool Model::unselectProjection( unsigned p )
    }
 }
 
-bool Model::isVertexSelected( unsigned v )
+bool Model::isVertexSelected( unsigned v ) const
 {
    if ( v < m_vertices.size() )
    {
@@ -373,7 +362,7 @@ bool Model::isVertexSelected( unsigned v )
    }
 }
 
-bool Model::isTriangleSelected( unsigned v )
+bool Model::isTriangleSelected( unsigned v ) const
 {
    if ( v < m_triangles.size() )
    {
@@ -385,7 +374,7 @@ bool Model::isTriangleSelected( unsigned v )
    }
 }
 
-bool Model::isGroupSelected( unsigned v )
+bool Model::isGroupSelected( unsigned v ) const
 {
    if ( v < m_groups.size() )
    {
@@ -397,7 +386,7 @@ bool Model::isGroupSelected( unsigned v )
    }
 }
 
-bool Model::isBoneJointSelected( unsigned j )
+bool Model::isBoneJointSelected( unsigned j ) const
 {
    if ( j < m_joints.size() )
    {
@@ -409,7 +398,7 @@ bool Model::isBoneJointSelected( unsigned j )
    }
 }
 
-bool Model::isPointSelected( unsigned p )
+bool Model::isPointSelected( unsigned p ) const
 {
    if ( p < m_points.size() )
    {
@@ -421,7 +410,7 @@ bool Model::isPointSelected( unsigned p )
    }
 }
 
-bool Model::isProjectionSelected( unsigned p )
+bool Model::isProjectionSelected( unsigned p ) const
 {
    if ( p < m_projections.size() )
    {
@@ -437,6 +426,13 @@ bool Model::selectVerticesInVolumeMatrix( bool select, const Matrix & viewMat, d
 {
    LOG_PROFILE();
 
+   if ( m_animationMode == Model::ANIMMODE_FRAME
+         && (m_currentAnim >= m_frameAnims.size() || 
+             m_currentFrame >= m_frameAnims[m_currentAnim]->m_frameData.size()) )
+   {
+      return false;
+   }
+
    beginSelectionDifference();
 
    if ( x1 > x2 )
@@ -491,6 +487,13 @@ bool Model::selectTrianglesInVolumeMatrix( bool select, const Matrix & viewMat,
 {
    LOG_PROFILE();
 
+   if ( m_animationMode == Model::ANIMMODE_FRAME
+         && (m_currentAnim >= m_frameAnims.size() || 
+             m_currentFrame >= m_frameAnims[m_currentAnim]->m_frameData.size()) )
+   {
+      return false;
+   }
+
    beginSelectionDifference();
 
    unsigned i;
@@ -748,25 +751,28 @@ next_triangle:
          found = false;
          for ( t = 0; t < m_triangles.size(); t++ )
          {
-            int count = 0;
-            for ( unsigned v = 0; v < 3; v++ )
+            if ( m_triangles[t]->m_visible )
             {
-               if ( m_vertices[ m_triangles[t]->m_vertexIndices[v] ]->m_marked2 )
-               {
-                  count++;
-               }
-            }
-
-            if ( count > 0 && 
-                  (count < 3 || m_triangles[t]->m_selected != select) ) 
-            {
-               found = true;
-
-               m_triangles[t]->m_selected = select;
-
+               int count = 0;
                for ( unsigned v = 0; v < 3; v++ )
                {
-                  m_vertices[ m_triangles[t]->m_vertexIndices[v] ]->m_marked2 = true;
+                  if ( m_vertices[ m_triangles[t]->m_vertexIndices[v] ]->m_marked2 )
+                  {
+                     count++;
+                  }
+               }
+
+               if ( count > 0 && 
+                     (count < 3 || m_triangles[t]->m_selected != select) ) 
+               {
+                  found = true;
+
+                  m_triangles[t]->m_selected = select;
+
+                  for ( unsigned v = 0; v < 3; v++ )
+                  {
+                     m_vertices[ m_triangles[t]->m_vertexIndices[v] ]->m_marked2 = true;
+                  }
                }
             }
          }
@@ -855,6 +861,13 @@ bool Model::selectPointsInVolumeMatrix( bool select, const Matrix & viewMat, dou
 {
    LOG_PROFILE();
 
+   if ( m_animationMode == Model::ANIMMODE_FRAME
+         && (m_currentAnim >= m_frameAnims.size() || 
+             m_currentFrame >= m_frameAnims[m_currentAnim]->m_frameData.size()) )
+   {
+      return false;
+   }
+
    beginSelectionDifference();
 
    if ( x1 > x2 )
@@ -949,7 +962,7 @@ bool Model::selectProjectionsInVolumeMatrix( bool select, const Matrix & viewMat
             // Because up and seam are vectors from pos, we can assume
             // that pos is at the origin. The math works out the same.
             double orig[3] = { 0, 0, 0 };
-            _calculateNormal( left.getVector(), orig, up.getVector(), seam.getVector() );
+            calculate_normal( left.getVector(), orig, up.getVector(), seam.getVector() );
 
             up.scale3( upMag );
             left.scale3( upMag );
@@ -1330,13 +1343,16 @@ void Model::selectTrianglesFromGroups()
 
    for ( unsigned g = 0; g < m_groups.size(); g++ )
    {
-      if ( m_groups[g]->m_selected )
+      Group * grp = m_groups[g];
+      if ( grp->m_selected )
       {
-         for ( unsigned t = 0; t < m_groups[g]->m_triangleIndices.size(); t++ )
+         for ( std::set<int>::const_iterator it = grp->m_triangleIndices.begin();
+               it != grp->m_triangleIndices.end();
+               ++it )
          {
-            if ( m_triangles[ m_groups[g]->m_triangleIndices[t] ]->m_visible )
+            if ( m_triangles[ *it ]->m_visible )
             {
-               m_triangles[ m_groups[g]->m_triangleIndices[t] ]->m_selected = true;
+               m_triangles[ *it ]->m_selected = true;
             }
          }
       }
@@ -1397,10 +1413,13 @@ void Model::selectGroupsFromTriangles( bool all )
 
    for ( unsigned g = 0; g < m_groups.size(); g++ )
    {
+      Group * grp = m_groups[g];
       unsigned count = 0;
-      for ( unsigned t = 0; t < m_groups[g]->m_triangleIndices.size(); t++ )
+      for ( std::set<int>::const_iterator it = grp->m_triangleIndices.begin();
+            it != grp->m_triangleIndices.end();
+            ++it )
       {
-         if ( m_triangles[ m_groups[g]->m_triangleIndices[t] ]->m_selected )
+         if ( m_triangles[ *it ]->m_selected )
          {
             count++;
          }
@@ -1410,22 +1429,22 @@ void Model::selectGroupsFromTriangles( bool all )
       {
          if ( count == m_groups[g]->m_triangleIndices.size() )
          {
-            m_groups[g]->m_selected = true;
+            grp->m_selected = true;
          }
          else
          {
-            m_groups[g]->m_selected = false;
+            grp->m_selected = false;
          }
       }
       else
       {
          if ( count > 0 )
          {
-            m_groups[g]->m_selected = true;
+            grp->m_selected = true;
          }
          else
          {
-            m_groups[g]->m_selected = false;
+            grp->m_selected = false;
          }
       }
    }
@@ -1650,7 +1669,7 @@ void Model::endSelectionDifference()
    }
 }
 
-void Model::getSelectedPositions( list< Position > & positions )
+void Model::getSelectedPositions( list< Position > & positions ) const
 {
    unsigned count;
    unsigned t;
@@ -1706,7 +1725,7 @@ void Model::getSelectedPositions( list< Position > & positions )
    }
 }
 
-void Model::getSelectedVertices( list<int> & vertices )
+void Model::getSelectedVertices( list<int> & vertices ) const
 {
    vertices.clear();
 
@@ -1719,7 +1738,7 @@ void Model::getSelectedVertices( list<int> & vertices )
    }
 }
 
-void Model::getSelectedTriangles( list<int> & triangles )
+void Model::getSelectedTriangles( list<int> & triangles ) const
 {
    triangles.clear();
 
@@ -1732,7 +1751,7 @@ void Model::getSelectedTriangles( list<int> & triangles )
    }
 }
 
-void Model::getSelectedGroups( list<int> & groups )
+void Model::getSelectedGroups( list<int> & groups ) const
 {
    groups.clear();
 
@@ -1745,7 +1764,7 @@ void Model::getSelectedGroups( list<int> & groups )
    }
 }
 
-void Model::getSelectedBoneJoints( list<int> & joints )
+void Model::getSelectedBoneJoints( list<int> & joints ) const
 {
    joints.clear();
 
@@ -1758,7 +1777,7 @@ void Model::getSelectedBoneJoints( list<int> & joints )
    }
 }
 
-void Model::getSelectedPoints( list<int> & points )
+void Model::getSelectedPoints( list<int> & points ) const
 {
    points.clear();
 
@@ -1771,7 +1790,7 @@ void Model::getSelectedPoints( list<int> & points )
    }
 }
 
-void Model::getSelectedProjections( list<int> & projections )
+void Model::getSelectedProjections( list<int> & projections ) const
 {
    projections.clear();
 
@@ -1784,7 +1803,7 @@ void Model::getSelectedProjections( list<int> & projections )
    }
 }
 
-unsigned Model::getSelectedVertexCount()
+unsigned Model::getSelectedVertexCount() const
 {
    unsigned c = m_vertices.size();
    unsigned count = 0;
@@ -1800,7 +1819,7 @@ unsigned Model::getSelectedVertexCount()
    return count;
 }
 
-unsigned Model::getSelectedTriangleCount()
+unsigned Model::getSelectedTriangleCount() const
 {
    unsigned c = m_triangles.size();
    unsigned count = 0;
@@ -1816,7 +1835,7 @@ unsigned Model::getSelectedTriangleCount()
    return count;
 }
 
-unsigned Model::getSelectedBoneJointCount()
+unsigned Model::getSelectedBoneJointCount() const
 {
    unsigned c = m_joints.size();
    unsigned count = 0;
@@ -1832,7 +1851,7 @@ unsigned Model::getSelectedBoneJointCount()
    return count;
 }
 
-unsigned Model::getSelectedPointCount()
+unsigned Model::getSelectedPointCount() const
 {
    unsigned c = m_points.size();
    unsigned count = 0;
@@ -1848,7 +1867,7 @@ unsigned Model::getSelectedPointCount()
    return count;
 }
 
-unsigned Model::getSelectedProjectionCount()
+unsigned Model::getSelectedProjectionCount() const
 {
    unsigned c = m_projections.size();
    unsigned count = 0;
@@ -1864,7 +1883,7 @@ unsigned Model::getSelectedProjectionCount()
    return count;
 }
 
-bool Model::parentJointSelected( int joint )
+bool Model::parentJointSelected( int joint ) const
 {
    while ( m_joints[joint]->m_parent >= 0 )
    {
@@ -1878,7 +1897,7 @@ bool Model::parentJointSelected( int joint )
    return false;
 }
 
-bool Model::directParentJointSelected( int joint )
+bool Model::directParentJointSelected( int joint ) const
 {
    int p = m_joints[joint]->m_parent;
    if ( p >= 0 )
@@ -1888,7 +1907,7 @@ bool Model::directParentJointSelected( int joint )
    return false;
 }
 
-bool Model::getSelectedBoundingRegion( double *x1, double *y1, double *z1, double *x2, double *y2, double *z2 )
+bool Model::getSelectedBoundingRegion( double *x1, double *y1, double *z1, double *x2, double *y2, double *z2 ) const
 {
    if ( x1 && y1 && z1 && x2 && y2 && z2 )
    {
