@@ -1,6 +1,6 @@
 /*  Misfit Model 3D
  * 
- *  Copyright (c) 2004-2005 Kevin Worcester
+ *  Copyright (c) 2004-2007 Kevin Worcester
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -62,17 +62,17 @@ class MU_NoOp : public Undo
       unsigned size() { return sizeof(MU_NoOp); };
 };
 
-class MU_TranslateSelectedVertices : public ModelUndo
+class MU_TranslateSelected : public ModelUndo
 {
    public:
-      MU_TranslateSelectedVertices();
-      virtual ~MU_TranslateSelectedVertices();
+      MU_TranslateSelected();
+      virtual ~MU_TranslateSelected();
 
       void undo( Model * );
       void redo( Model * );
       bool combine( Undo * );
 
-      unsigned size() { return sizeof(MU_TranslateSelectedVertices); };
+      unsigned size() { return sizeof(MU_TranslateSelected); };
 
       void setMatrix( const Matrix & rhs );
 
@@ -82,17 +82,17 @@ class MU_TranslateSelectedVertices : public ModelUndo
       Matrix m_matrix;
 };
 
-class MU_RotateSelectedVerticesOnPoint : public ModelUndo
+class MU_RotateSelected : public ModelUndo
 {
    public:
-      MU_RotateSelectedVerticesOnPoint();
-      virtual ~MU_RotateSelectedVerticesOnPoint();
+      MU_RotateSelected();
+      virtual ~MU_RotateSelected();
 
       void undo( Model * );
       void redo( Model * );
       bool combine( Undo * );
 
-      unsigned size() { return sizeof(MU_RotateSelectedVerticesOnPoint); };
+      unsigned size() { return sizeof(MU_RotateSelected); };
 
       void setMatrixPoint( const Matrix & rhs, double * point );
       Matrix getMatrix() const { return m_matrix; };
@@ -115,11 +115,13 @@ class MU_ApplyMatrix : public ModelUndo
 
       unsigned size() { return sizeof(MU_ApplyMatrix); };
 
-      void setMatrix( const Matrix & rhs );
+      void setMatrix( const Matrix & mat, Model::OperationScopeE scope, bool animations );
       Matrix getMatrix() const { return m_matrix; };
 
    private:
       Matrix m_matrix;
+      Model::OperationScopeE m_scope;
+      bool m_animations;
 };
 
 class MU_SelectionMode : public ModelUndo
@@ -134,17 +136,17 @@ class MU_SelectionMode : public ModelUndo
 
       unsigned size();
 
-      void setSelectionMode( Model::SelectionMode mode, Model::SelectionMode oldMode ) { m_mode = mode; m_oldMode = oldMode; };
+      void setSelectionMode( Model::SelectionModeE mode, Model::SelectionModeE oldMode ) { m_mode = mode; m_oldMode = oldMode; };
 
    private:
-      Model::SelectionMode m_mode;
-      Model::SelectionMode m_oldMode;
+      Model::SelectionModeE m_mode;
+      Model::SelectionModeE m_oldMode;
 };
 
 class MU_Select : public ModelUndo
 {
    public:
-      MU_Select( Model::SelectionMode mode );
+      MU_Select( Model::SelectionModeE mode );
       virtual ~MU_Select();
 
       void undo( Model * );
@@ -153,7 +155,7 @@ class MU_Select : public ModelUndo
 
       unsigned size();
 
-      Model::SelectionMode getSelectionMode() const { return m_mode; };
+      Model::SelectionModeE getSelectionMode() const { return m_mode; };
 
       void setSelectionDifference( int number, bool selected, bool oldSelected );
       void toggle( int number );
@@ -161,24 +163,24 @@ class MU_Select : public ModelUndo
       unsigned diffCount() { return m_diff.size(); };
 
    private:
-      typedef struct SelectionDifference_t
+      typedef struct _SelectionDifference_t
       {
          int  number;
          bool selected;
          bool oldSelected;
-         bool operator< ( const struct SelectionDifference_t & rhs ) const
+         bool operator< ( const struct _SelectionDifference_t & rhs ) const
          {
             return ( this->number < rhs.number );
          }
-         bool operator== ( const struct SelectionDifference_t & rhs ) const
+         bool operator== ( const struct _SelectionDifference_t & rhs ) const
          {
             return ( this->number == rhs.number );
          }
-      } SelectionDifference;
+      } SelectionDifferenceT;
 
-      typedef sorted_list< SelectionDifference > SelectionDifferenceList;
+      typedef sorted_list< SelectionDifferenceT > SelectionDifferenceList;
 
-      Model::SelectionMode m_mode;
+      Model::SelectionModeE m_mode;
       SelectionDifferenceList m_diff;
       
 };
@@ -186,7 +188,7 @@ class MU_Select : public ModelUndo
 class MU_Hide : public ModelUndo
 {
    public:
-      MU_Hide( Model::SelectionMode mode );
+      MU_Hide( Model::SelectionModeE mode );
       virtual ~MU_Hide();
 
       void undo( Model * );
@@ -195,20 +197,20 @@ class MU_Hide : public ModelUndo
 
       unsigned size();
 
-      Model::SelectionMode getSelectionMode() const { return m_mode; };
+      Model::SelectionModeE getSelectionMode() const { return m_mode; };
 
       void setHideDifference( int number, bool visible );
 
    private:
-      typedef struct HideDifference_t
+      typedef struct _HideDifference_t
       {
          int  number;
          bool visible;
-      } HideDifference;
+      } HideDifferenceT;
 
-      typedef list< HideDifference > HideDifferenceList;
+      typedef list< HideDifferenceT > HideDifferenceList;
 
-      Model::SelectionMode m_mode;
+      Model::SelectionModeE m_mode;
       HideDifferenceList m_diff;
 };
 
@@ -230,11 +232,11 @@ class MU_InvertNormal : public ModelUndo
       list<int> m_triangles;
 };
 
-class MU_MoveVertex : public ModelUndo
+class MU_MovePrimitive : public ModelUndo
 {
    public:
-      MU_MoveVertex();
-      virtual ~MU_MoveVertex();
+      MU_MovePrimitive();
+      virtual ~MU_MovePrimitive();
 
       void undo( Model * );
       void redo( Model * );
@@ -242,14 +244,21 @@ class MU_MoveVertex : public ModelUndo
 
       unsigned size();
 
-      void addVertex( int v, double x, double y, double z,
-            double oldx, double oldy, double oldz );
-      void addBoneJoint( int j, double x, double y, double z,
+      typedef enum _MoveType_e
+      {
+          MT_Vertex,
+          MT_Joint,
+          MT_Point,
+          MT_Projection
+      };
+      typedef enum _MoveType_e MoveTypeE;
+
+      void addMovePrimitive( MoveTypeE, int i, double x, double y, double z,
             double oldx, double oldy, double oldz );
 
    private:
 
-      typedef struct MoveVertex_t
+      typedef struct _MovePrimitive_t
       {
          int number;
          double x;
@@ -258,22 +267,74 @@ class MU_MoveVertex : public ModelUndo
          double oldx;
          double oldy;
          double oldz;
-         bool   isJoint;
+         MoveTypeE type;
 
-         bool operator< ( const struct MoveVertex_t & rhs ) const
+         bool operator< ( const struct _MovePrimitive_t & rhs ) const
          {
             return ( this->number < rhs.number || 
-                  ( this->number == rhs.number && this->isJoint == false && rhs.isJoint == true ) 
+                  ( this->number == rhs.number && this->type < rhs.type ) 
                   );
          };
-         bool operator== ( const struct MoveVertex_t & rhs ) const
+         bool operator== ( const struct _MovePrimitive_t & rhs ) const
          {
-            return ( this->number == rhs.number && this->isJoint == rhs.isJoint );
+            return ( this->number == rhs.number && this->type == rhs.type );
          };
-      } MoveVertex;
-      typedef sorted_list<MoveVertex> MoveVertexList;
+      } MovePrimitiveT;
+      typedef sorted_list<MovePrimitiveT> MovePrimitiveList;
 
-      MoveVertexList m_vertices;
+      MovePrimitiveList m_objects;
+};
+
+class MU_SetPointRotation : public ModelUndo
+{
+   public:
+      MU_SetPointRotation();
+      virtual ~MU_SetPointRotation();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setPointRotation( int p, double x, double y, double z,
+            double oldx, double oldy, double oldz );
+
+   private:
+
+      int number;
+      double x;
+      double y;
+      double z;
+      double oldx;
+      double oldy;
+      double oldz;
+};
+
+class MU_SetPointTranslation : public ModelUndo
+{
+   public:
+      MU_SetPointTranslation();
+      virtual ~MU_SetPointTranslation();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setPointTranslation( int p, double x, double y, double z,
+            double oldx, double oldy, double oldz );
+
+   private:
+
+      int number;
+      double x;
+      double y;
+      double z;
+      double oldx;
+      double oldy;
+      double oldz;
 };
 
 class MU_SetTexture : public ModelUndo
@@ -291,13 +352,13 @@ class MU_SetTexture : public ModelUndo
       void setTexture( unsigned groupNumber, int newTexture, int oldTexture );
 
    private:
-      typedef struct SetTexture_t
+      typedef struct _SetTexture_t
       {
          unsigned groupNumber;
          int newTexture;
          int oldTexture;
-      } SetTexture;
-      typedef list<SetTexture> SetTextureList;
+      } SetTextureT;
+      typedef list<SetTextureT> SetTextureList;
 
       SetTextureList m_list;
 };
@@ -319,12 +380,12 @@ class MU_AddVertex : public ModelUndo
       void addVertex( unsigned index, Model::Vertex * vertex );
 
    private:
-      typedef struct AddVertex_t
+      typedef struct _AddVertex_t
       {
          unsigned index;
          Model::Vertex * vertex;
-      } AddVertex;
-      typedef list<AddVertex> AddVertexList;
+      } AddVertexT;
+      typedef list<AddVertexT> AddVertexList;
 
       AddVertexList m_list;
 };
@@ -346,12 +407,12 @@ class MU_AddTriangle : public ModelUndo
       void addTriangle( unsigned index, Model::Triangle * );
 
    private:
-      typedef struct AddTriangle_t
+      typedef struct _AddTriangle_t
       {
          unsigned index;
          Model::Triangle * triangle;
-      } AddTriangle;
-      typedef list<AddTriangle> AddTriangleList;
+      } AddTriangleT;
+      typedef list<AddTriangleT> AddTriangleList;
 
       AddTriangleList m_list;
 };
@@ -373,12 +434,12 @@ class MU_AddGroup : public ModelUndo
       void addGroup( unsigned index, Model::Group * );
 
    private:
-      typedef struct AddGroup_t
+      typedef struct _AddGroup_t
       {
          unsigned index;
          Model::Group * group;
-      } AddGroup;
-      typedef list<AddGroup> AddGroupList;
+      } AddGroupT;
+      typedef list<AddGroupT> AddGroupList;
 
       AddGroupList m_list;
 };
@@ -400,12 +461,12 @@ class MU_AddTexture : public ModelUndo
       void addTexture( unsigned index, Model::Material * );
 
    private:
-      typedef struct AddTexture_t
+      typedef struct _AddTexture_t
       {
          unsigned index;
          Model::Material * texture;
-      } AddTexture;
-      typedef list<AddTexture> AddTextureList;
+      } AddTextureT;
+      typedef list<AddTextureT> AddTextureList;
 
       AddTextureList m_list;
 };
@@ -425,7 +486,7 @@ class MU_SetTextureCoords : public ModelUndo
       void addTextureCoords( unsigned triangle, unsigned vertexIndex, float s, float t, float oldS, float oldT );
 
    private:
-      typedef struct SetTextureCoords_t
+      typedef struct _SetTextureCoords_t
       {
          unsigned triangle;
          unsigned vertexIndex;
@@ -433,18 +494,18 @@ class MU_SetTextureCoords : public ModelUndo
          float s;
          float oldT;
          float oldS;
-         bool operator< ( const struct SetTextureCoords_t & rhs ) const
+         bool operator< ( const struct _SetTextureCoords_t & rhs ) const
          {
             return ( this->triangle < rhs.triangle
                   || (this->triangle == rhs.triangle && this->vertexIndex < rhs.vertexIndex ) );
          };
-         bool operator == ( const struct SetTextureCoords_t & rhs ) const
+         bool operator == ( const struct _SetTextureCoords_t & rhs ) const
          {
             return ( this->triangle == rhs.triangle && this->vertexIndex == rhs.vertexIndex );
          };
-      } SetTextureCoords;
+      } SetTextureCoordsT;
 
-      typedef sorted_list<SetTextureCoords> STCList;
+      typedef sorted_list<SetTextureCoordsT> STCList;
 
       STCList m_list;
 };
@@ -464,12 +525,12 @@ class MU_AddToGroup : public ModelUndo
       void addToGroup( unsigned groupNum, unsigned triangleNum );
 
    private:
-      typedef struct _AddToGroup
+      typedef struct _AddToGroup_t
       {
          unsigned groupNum;
          unsigned triangleNum;
-      } AddToGroup;
-      typedef list<AddToGroup> AddToGroupList;
+      } AddToGroupT;
+      typedef list<AddToGroupT> AddToGroupList;
 
       AddToGroupList m_list;
 };
@@ -489,12 +550,12 @@ class MU_RemoveFromGroup : public ModelUndo
       void removeFromGroup( unsigned groupNum, unsigned triangleNum );
 
    private:
-      typedef struct _RemoveFromGroup
+      typedef struct _RemoveFromGroup_t
       {
          unsigned groupNum;
          unsigned triangleNum;
-      } RemoveFromGroup;
-      typedef list<RemoveFromGroup> RemoveFromGroupList;
+      } RemoveFromGroupT;
+      typedef list<RemoveFromGroupT> RemoveFromGroupList;
 
       RemoveFromGroupList m_list;
 };
@@ -516,12 +577,12 @@ class MU_DeleteTriangle : public ModelUndo
       void deleteTriangle( unsigned triangleNum, Model::Triangle * triangle );
 
    private:
-      typedef struct DeleteTriangle_t
+      typedef struct _DeleteTriangle_t
       {
          unsigned triangleNum;
          Model::Triangle * triangle;
-      } DeleteTriangle;
-      typedef list<DeleteTriangle> DeleteTriangleList;
+      } DeleteTriangleT;
+      typedef list<DeleteTriangleT> DeleteTriangleList;
 
       DeleteTriangleList m_list;
 };
@@ -543,12 +604,12 @@ class MU_DeleteVertex : public ModelUndo
       void deleteVertex( unsigned vertexNum, Model::Vertex * vertex );
 
    private:
-      typedef struct DeleteVertex_t
+      typedef struct _DeleteVertex_t
       {
          unsigned vertexNum;
          Model::Vertex * vertex;
-      } DeleteVertex;
-      typedef list<DeleteVertex> DeleteVertexList;
+      } DeleteVertexT;
+      typedef list<DeleteVertexT> DeleteVertexList;
 
       DeleteVertexList m_list;
 };
@@ -574,6 +635,48 @@ class MU_DeleteBoneJoint : public ModelUndo
       Model::Joint * m_joint;
 };
 
+class MU_DeletePoint : public ModelUndo
+{
+   public:
+      MU_DeletePoint();
+      virtual ~MU_DeletePoint();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      void undoRelease();
+
+      unsigned size();
+
+      void deletePoint( unsigned pointNum, Model::Point * point );
+
+   private:
+      unsigned       m_pointNum;
+      Model::Point * m_point;
+};
+
+class MU_DeleteProjection : public ModelUndo
+{
+   public:
+      MU_DeleteProjection();
+      virtual ~MU_DeleteProjection();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      void undoRelease();
+
+      unsigned size();
+
+      void deleteProjection( unsigned projNum, Model::TextureProjection * proj );
+
+   private:
+      unsigned       m_projNum;
+      Model::TextureProjection * m_proj;
+};
+
 class MU_AddBoneJoint : public ModelUndo
 {
    public:
@@ -595,6 +698,48 @@ class MU_AddBoneJoint : public ModelUndo
       Model::Joint * m_joint;
 };
 
+class MU_AddPoint : public ModelUndo
+{
+   public:
+      MU_AddPoint();
+      virtual ~MU_AddPoint();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      void redoRelease();
+
+      unsigned size();
+
+      void addPoint( unsigned pointNum, Model::Point * point );
+
+   private:
+      unsigned       m_pointNum;
+      Model::Point * m_point;
+};
+
+class MU_AddProjection : public ModelUndo
+{
+   public:
+      MU_AddProjection();
+      virtual ~MU_AddProjection();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      void redoRelease();
+
+      unsigned size();
+
+      void addProjection( unsigned pointNum, Model::TextureProjection * point );
+
+   private:
+      unsigned       m_projNum;
+      Model::TextureProjection * m_proj;
+};
+
 class MU_DeleteGroup : public ModelUndo
 {
    public:
@@ -612,12 +757,12 @@ class MU_DeleteGroup : public ModelUndo
       void deleteGroup( unsigned groupNum, Model::Group * group );
 
    private:
-      typedef struct DeleteGroup_t
+      typedef struct _DeleteGroup_t
       {
          unsigned groupNum;
          Model::Group * group;
-      } DeleteGroup;
-      typedef list<DeleteGroup> DeleteGroupList;
+      } DeleteGroupT;
+      typedef list<DeleteGroupT> DeleteGroupList;
 
       DeleteGroupList m_list;
 };
@@ -639,12 +784,12 @@ class MU_DeleteTexture : public ModelUndo
       void deleteTexture( unsigned textureNum, Model::Material * texture );
 
    private:
-      typedef struct DeleteTexture_t
+      typedef struct _DeleteTexture_t
       {
          unsigned textureNum;
          Model::Material * texture;
-      } DeleteTexture;
-      typedef list<DeleteTexture> DeleteTextureList;
+      } DeleteTextureT;
+      typedef list<DeleteTextureT> DeleteTextureList;
 
       DeleteTextureList m_list;
 };
@@ -652,14 +797,15 @@ class MU_DeleteTexture : public ModelUndo
 class MU_SetLightProperties : public ModelUndo
 {
    public:
-      typedef enum
+      enum _LightType_e
       {
          LightAmbient  = 0,
          LightDiffuse,
          LightSpecular,
          LightEmissive,
          LightTypeMax  
-      } LightType;
+      };
+      typedef enum _LightType_e LightTypeE;
 
       MU_SetLightProperties();
       virtual ~MU_SetLightProperties();
@@ -670,20 +816,20 @@ class MU_SetLightProperties : public ModelUndo
 
       unsigned size();
 
-      void setLightProperties( unsigned textureNum, LightType type, const float * newLight, const float * oldLight );
+      void setLightProperties( unsigned textureNum, LightTypeE type, const float * newLight, const float * oldLight );
 
    private:
-      typedef struct LightProperties_t
+      typedef struct _LightProperties_t
       {
          unsigned textureNum;
          float newLight[ LightTypeMax ][4];
          float oldLight[ LightTypeMax ][4];
          bool  isSet[ LightTypeMax ];
-      } LightProperties;
+      } LightPropertiesT;
 
-      typedef list<LightProperties> LightPropertiesList;
+      typedef list<LightPropertiesT> LightPropertiesList;
 
-      list<LightProperties> m_list;
+      list<LightPropertiesT> m_list;
 };
 
 class MU_SetShininess : public ModelUndo
@@ -702,16 +848,16 @@ class MU_SetShininess : public ModelUndo
       void setShininess( unsigned textureNum, const float & newValue, const float & oldValue );
 
    private:
-      typedef struct Shininess_t
+      typedef struct _Shininess_t
       {
          unsigned textureNum;
          float oldValue;
          float newValue;
-      } Shininess;
+      } ShininessT;
 
-      typedef list<Shininess> ShininessList;
+      typedef list<ShininessT> ShininessList;
 
-      list<Shininess> m_list;
+      list<ShininessT> m_list;
 };
 
 class MU_SetTextureName : public ModelUndo
@@ -753,14 +899,14 @@ class MU_SetTriangleVertices : public ModelUndo
             unsigned oldV1, unsigned oldV2, unsigned oldV3 );
 
    private:
-      typedef struct TriangleVertices_t
+      typedef struct _TriangleVertices_t
       {
          unsigned triangleNum;
          unsigned newVertices[3];
          unsigned oldVertices[3];
-      } TriangleVertices;
+      } TriangleVerticesT;
 
-      typedef list<TriangleVertices> TriangleVerticesList;
+      typedef list<TriangleVerticesT> TriangleVerticesList;
 
       TriangleVerticesList m_list;
 };
@@ -797,15 +943,15 @@ class MU_SubdivideTriangle : public ModelUndo
 
    private:
 
-      typedef struct SubdivideTriangle_t
+      typedef struct _SubdivideTriangle_t
       {
          unsigned a;
          unsigned b;
          unsigned c;
          unsigned d;
-      } SubdivideTriangle;
+      } SubdivideTriangleT;
 
-      typedef list<SubdivideTriangle> SubdivideTriangleList;
+      typedef list<SubdivideTriangleT> SubdivideTriangleList;
 
       SubdivideTriangleList m_list;
       list<unsigned> m_vlist;
@@ -823,12 +969,12 @@ class MU_ChangeAnimState : public ModelUndo
 
       unsigned size();
 
-      void setState( Model::AnimationMode newMode, Model::AnimationMode oldMode, unsigned anim, unsigned frame );
+      void setState( Model::AnimationModeE newMode, Model::AnimationModeE oldMode, unsigned anim, unsigned frame );
 
    protected:
 
-      Model::AnimationMode m_newMode;
-      Model::AnimationMode m_oldMode;
+      Model::AnimationModeE m_newMode;
+      Model::AnimationModeE m_oldMode;
       unsigned m_anim;
       unsigned m_frame;
 };
@@ -846,11 +992,11 @@ class MU_SetAnimName : public ModelUndo
 
       unsigned size();
 
-      void setName( Model::AnimationMode mode, unsigned animNum, const char * newName, const char * oldName );
+      void setName( Model::AnimationModeE mode, unsigned animNum, const char * newName, const char * oldName );
 
    private:
 
-      Model::AnimationMode m_mode;
+      Model::AnimationModeE m_mode;
       unsigned m_animNum;
       string   m_newName;
       string   m_oldName;
@@ -869,11 +1015,11 @@ class MU_SetAnimFrameCount : public ModelUndo
 
       unsigned size();
 
-      void setAnimFrameCount( Model::AnimationMode m, unsigned animNum, unsigned newCount, unsigned oldCount );
+      void setAnimFrameCount( Model::AnimationModeE m, unsigned animNum, unsigned newCount, unsigned oldCount );
 
    private:
 
-      Model::AnimationMode m_mode;
+      Model::AnimationModeE m_mode;
       unsigned m_animNum;
       unsigned m_newCount;
       unsigned m_oldCount;
@@ -892,11 +1038,11 @@ class MU_SetAnimFPS : public ModelUndo
 
       unsigned size();
 
-      void setFPS( Model::AnimationMode mode, unsigned animNum, double newFps, double oldFps );
+      void setFPS( Model::AnimationModeE mode, unsigned animNum, double newFps, double oldFps );
 
    private:
 
-      Model::AnimationMode m_mode;
+      Model::AnimationModeE m_mode;
       unsigned m_animNum;
       double   m_newFPS;
       double   m_oldFPS;
@@ -921,7 +1067,7 @@ class MU_SetAnimKeyframe : public ModelUndo
 
    private:
 
-      typedef struct SetKeyframe_t
+      typedef struct _SetKeyframe_t
       {
          int number;
          bool isNew;
@@ -932,16 +1078,16 @@ class MU_SetAnimKeyframe : public ModelUndo
          double oldy;
          double oldz;
 
-         bool operator< ( const struct SetKeyframe_t & rhs ) const
+         bool operator< ( const struct _SetKeyframe_t & rhs ) const
          {
             return ( this->number < rhs.number );
          };
-         bool operator== ( const struct SetKeyframe_t & rhs ) const
+         bool operator== ( const struct _SetKeyframe_t & rhs ) const
          {
             return ( this->number == rhs.number );
          };
-      } SetKeyframe;
-      typedef sorted_list<SetKeyframe> SetKeyframeList;
+      } SetKeyFrameT;
+      typedef sorted_list<SetKeyFrameT> SetKeyframeList;
 
       SetKeyframeList m_keyframes;
       unsigned        m_anim;
@@ -995,6 +1141,72 @@ class MU_SetJointName : public ModelUndo
       string   m_oldName;
 };
 
+class MU_SetPointName : public ModelUndo
+{
+   public:
+
+      MU_SetPointName();
+      virtual ~MU_SetPointName();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setName( unsigned point, const char * newName, const char * oldName );
+
+   private:
+
+      unsigned m_point;
+      string   m_newName;
+      string   m_oldName;
+};
+
+class MU_SetProjectionName : public ModelUndo
+{
+   public:
+
+      MU_SetProjectionName();
+      virtual ~MU_SetProjectionName();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setName( unsigned projection, const char * newName, const char * oldName );
+
+   private:
+
+      unsigned m_projection;
+      string   m_newName;
+      string   m_oldName;
+};
+
+class MU_SetProjectionType : public ModelUndo
+{
+   public:
+
+      MU_SetProjectionType();
+      virtual ~MU_SetProjectionType();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setType( unsigned projection, int newType, int oldType );
+
+   private:
+
+      unsigned m_projection;
+      int   m_newType;
+      int   m_oldType;
+};
+
 class MU_MoveFrameVertex : public ModelUndo
 {
    public:
@@ -1014,7 +1226,7 @@ class MU_MoveFrameVertex : public ModelUndo
 
    private:
 
-      typedef struct MoveFrameVertex_t
+      typedef struct _MoveFrameVertex_t
       {
          int number;
          double x;
@@ -1024,22 +1236,112 @@ class MU_MoveFrameVertex : public ModelUndo
          double oldy;
          double oldz;
 
-         bool operator< ( const struct MoveFrameVertex_t & rhs ) const
+         bool operator< ( const struct _MoveFrameVertex_t & rhs ) const
          {
             return ( this->number < rhs.number );
          };
-         bool operator== ( const struct MoveFrameVertex_t & rhs ) const
+         bool operator== ( const struct _MoveFrameVertex_t & rhs ) const
          {
             return ( this->number == rhs.number );
          };
-      } MoveFrameVertex;
-      typedef sorted_list<MoveFrameVertex> MoveFrameVertexList;
+      } MoveFrameVertexT;
+      typedef sorted_list<MoveFrameVertexT> MoveFrameVertexList;
 
       MoveFrameVertexList m_vertices;
       unsigned            m_anim;
       unsigned            m_frame;
 };
 
+class MU_MoveFramePoint : public ModelUndo
+{
+   public:
+      MU_MoveFramePoint();
+      virtual ~MU_MoveFramePoint();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setAnimationData( const unsigned & anim, const unsigned & frame  );
+
+      void addPoint( int p, double x, double y, double z,
+            double oldx, double oldy, double oldz );
+
+   private:
+
+      typedef struct _MoveFramePoint_t
+      {
+         int number;
+         double x;
+         double y;
+         double z;
+         double oldx;
+         double oldy;
+         double oldz;
+
+         bool operator< ( const struct _MoveFramePoint_t & rhs ) const
+         {
+            return ( this->number < rhs.number );
+         };
+         bool operator== ( const struct _MoveFramePoint_t & rhs ) const
+         {
+            return ( this->number == rhs.number );
+         };
+      } MoveFramePointT;
+      typedef sorted_list<MoveFramePointT> MoveFramePointList;
+
+      MoveFramePointList  m_points;
+      unsigned            m_anim;
+      unsigned            m_frame;
+};
+
+class MU_RotateFramePoint : public ModelUndo
+{
+   public:
+      MU_RotateFramePoint();
+      virtual ~MU_RotateFramePoint();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setAnimationData( const unsigned & anim, const unsigned & frame  );
+
+      void addPointRotation( int p, double x, double y, double z,
+            double oldx, double oldy, double oldz );
+
+   private:
+
+      typedef struct _RotateFramePoint_t
+      {
+         int number;
+         double x;
+         double y;
+         double z;
+         double oldx;
+         double oldy;
+         double oldz;
+
+         bool operator< ( const struct _RotateFramePoint_t & rhs ) const
+         {
+            return ( this->number < rhs.number );
+         };
+         bool operator== ( const struct _RotateFramePoint_t & rhs ) const
+         {
+            return ( this->number == rhs.number );
+         };
+      } RotateFramePointT;
+      typedef sorted_list<RotateFramePointT> RotateFramePointList;
+
+      RotateFramePointList  m_points;
+      unsigned            m_anim;
+      unsigned            m_frame;
+};
+
 class MU_AddFrameAnimFrame : public ModelUndo
 {
    public:
@@ -1055,18 +1357,18 @@ class MU_AddFrameAnimFrame : public ModelUndo
       unsigned size();
 
       void setAnimationData( const unsigned & anim );
-      void addFrame( const unsigned & frame, Model::FrameAnimVertexList * list );
+      void addFrame( const unsigned & frame, Model::FrameAnimData * data );
 
    private:
-      typedef struct AddFrame_t
+      typedef struct _AddFrame_t
       {
          unsigned frame;
-         Model::FrameAnimVertexList * list;
-      } AddFrame;
-      typedef list<AddFrame> FrameVertexList;
+         Model::FrameAnimData * data;
+      } AddFrameT;
+      typedef list<AddFrameT> FrameDataList;
 
-      FrameVertexList m_list;
-      unsigned        m_anim;
+      FrameDataList m_list;
+      unsigned      m_anim;
 };
 
 class MU_DeleteFrameAnimFrame : public ModelUndo
@@ -1084,20 +1386,63 @@ class MU_DeleteFrameAnimFrame : public ModelUndo
       unsigned size();
 
       void setAnimationData( const unsigned & anim );
-      void deleteFrame( const unsigned & frame, Model::FrameAnimVertexList * list );
+      void deleteFrame( const unsigned & frame, Model::FrameAnimData * data );
 
    private:
-      typedef struct DeleteFrame_t
+      typedef struct _DeleteFrame_t
       {
          unsigned frame;
-         Model::FrameAnimVertexList * list;
-      } DeleteFrame;
-      typedef list<DeleteFrame> FrameVertexList;
+         Model::FrameAnimData * data;
+      } DeleteFrameT;
+      typedef list<DeleteFrameT> FrameDataList;
 
-      FrameVertexList m_list;
+      FrameDataList    m_list;
       unsigned         m_anim;
 };
 
+class MU_SetPositionInfluence : public ModelUndo
+{
+   public:
+      MU_SetPositionInfluence();
+      virtual ~MU_SetPositionInfluence();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setPositionInfluence( bool isAdd, const Model::Position & pos, unsigned index, const Model::InfluenceT & influence );
+
+   private:
+      bool m_isAdd;
+      Model::Position m_pos;
+      unsigned m_index;
+      Model::InfluenceT m_influence;
+};
+
+class MU_UpdatePositionInfluence : public ModelUndo
+{
+   public:
+      MU_UpdatePositionInfluence();
+      virtual ~MU_UpdatePositionInfluence();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void updatePositionInfluence( const Model::Position & pos, 
+            const Model::InfluenceT & newInf,
+            const Model::InfluenceT & oldInf );
+
+   private:
+      Model::Position m_pos;
+      Model::InfluenceT m_newInf;
+      Model::InfluenceT m_oldInf;
+};
+
 class MU_SetVertexBoneJoint : public ModelUndo
 {
    public:
@@ -1114,17 +1459,71 @@ class MU_SetVertexBoneJoint : public ModelUndo
             const int & newBone, const int & oldBone );
 
    private:
-      typedef struct SetJoint_t
+      typedef struct _SetJoint_t
       {
          unsigned vertex;
          int newBone;
          int oldBone;
-      } SetJoint;
-      typedef list<SetJoint> SetJointList;
+      } SetJointT;
+      typedef list<SetJointT> SetJointList;
 
       SetJointList m_list;
 };
 
+class MU_SetPointBoneJoint : public ModelUndo
+{
+   public:
+      MU_SetPointBoneJoint();
+      virtual ~MU_SetPointBoneJoint();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setPointBoneJoint( const unsigned & point, 
+            const int & newBone, const int & oldBone );
+
+   private:
+      typedef struct _SetJoint_t
+      {
+         unsigned point;
+         int newBone;
+         int oldBone;
+      } SetJointT;
+      typedef list<SetJointT> SetJointList;
+
+      SetJointList m_list;
+};
+
+class MU_SetTriangleProjection : public ModelUndo
+{
+   public:
+      MU_SetTriangleProjection();
+      virtual ~MU_SetTriangleProjection();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setTriangleProjection( const unsigned & vertex, 
+            const int & newProj, const int & oldProj );
+
+   private:
+      typedef struct _SetProjection_t
+      {
+         unsigned triangle;
+         int newProj;
+         int oldProj;
+      } SetProjectionT;
+      typedef list<SetProjectionT> SetProjectionList;
+
+      SetProjectionList m_list;
+};
+
 class MU_AddAnimation : public ModelUndo
 {
    public:
@@ -1233,6 +1632,68 @@ class MU_SetJointTranslation : public ModelUndo
       double   m_oldTranslation[3];
 };
 
+class MU_SetProjectionUp : public ModelUndo
+{
+   public:
+      MU_SetProjectionUp();
+      virtual ~MU_SetProjectionUp();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * ) { return false; };
+
+      unsigned size() { return sizeof(MU_SetProjectionUp); };
+
+      void setProjectionUp( const unsigned & proj, const double * newUp, const double * oldUp );
+
+   protected:
+      unsigned m_proj;
+      double   m_newUp[3];
+      double   m_oldUp[3];
+};
+
+class MU_SetProjectionSeam : public ModelUndo
+{
+   public:
+      MU_SetProjectionSeam();
+      virtual ~MU_SetProjectionSeam();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * ) { return false; };
+
+      unsigned size() { return sizeof(MU_SetProjectionSeam); };
+
+      void setProjectionSeam( const unsigned & proj, const double * newSeam, const double * oldSeam );
+
+   protected:
+      unsigned m_proj;
+      double   m_newSeam[3];
+      double   m_oldSeam[3];
+};
+
+class MU_SetProjectionRange : public ModelUndo
+{
+   public:
+      MU_SetProjectionRange();
+      virtual ~MU_SetProjectionRange();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * ) { return false; };
+
+      unsigned size() { return sizeof(MU_SetProjectionRange); };
+
+      void setProjectionRange( const unsigned & proj, 
+            double newXMin, double newYMin, double newXMax, double newYMax,
+            double oldXMin, double oldYMin, double oldXMax, double oldYMax );
+
+   protected:
+      unsigned m_proj;
+      double   m_newRange[4];
+      double   m_oldRange[4];
+};
+
 class MU_SetGroupSmooth : public ModelUndo
 {
    public:
@@ -1249,13 +1710,13 @@ class MU_SetGroupSmooth : public ModelUndo
             const uint8_t & newSmooth, const uint8_t & oldSmooth );
 
    private:
-      typedef struct SetSmooth_t
+      typedef struct _SetSmooth_t
       {
          unsigned group;
          uint8_t newSmooth;
          uint8_t oldSmooth;
-      } SetSmooth;
-      typedef list<SetSmooth> SetSmoothList;
+      } SetSmoothT;
+      typedef list<SetSmoothT> SetSmoothList;
 
       SetSmoothList m_list;
 };
@@ -1276,13 +1737,13 @@ class MU_SetGroupAngle : public ModelUndo
             const uint8_t & newAngle, const uint8_t & oldAngle );
 
    private:
-      typedef struct SetAngle_t
+      typedef struct _SetAngle_t
       {
          unsigned group;
          uint8_t newAngle;
          uint8_t oldAngle;
-      } SetAngle;
-      typedef list<SetAngle> SetAngleList;
+      } SetAngleT;
+      typedef list<SetAngleT> SetAngleList;
 
       SetAngleList m_list;
 };
@@ -1322,11 +1783,11 @@ class MU_MoveAnimation : public ModelUndo
 
       unsigned size();
 
-      void moveAnimation( const Model::AnimationMode & mode, const unsigned & oldIndex, const unsigned & newIndex );
+      void moveAnimation( const Model::AnimationModeE & mode, const unsigned & oldIndex, const unsigned & newIndex );
 
    private:
 
-      Model::AnimationMode m_mode;
+      Model::AnimationModeE m_mode;
       unsigned m_oldIndex;
       unsigned m_newIndex;
 };
@@ -1352,6 +1813,27 @@ class MU_SetFrameAnimVertexCount : public ModelUndo
       unsigned m_oldCount;
 };
 
+class MU_SetFrameAnimPointCount : public ModelUndo
+{
+   public:
+
+      MU_SetFrameAnimPointCount();
+      virtual ~MU_SetFrameAnimPointCount();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * ) { return false; };
+
+      unsigned size();
+
+      void setCount( const unsigned & newCount, const unsigned & oldCount );
+
+   private:
+
+      unsigned m_newCount;
+      unsigned m_oldCount;
+};
+
 class MU_SetMaterialClamp : public ModelUndo
 {
    public:
@@ -1374,6 +1856,27 @@ class MU_SetMaterialClamp : public ModelUndo
       bool m_newClamp;
 };
 
+class MU_SetMaterialTexture : public ModelUndo
+{
+   public:
+      MU_SetMaterialTexture();
+      virtual ~MU_SetMaterialTexture();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void setMaterialTexture( const unsigned & material, 
+            Texture * newTexture, Texture * oldTexture );
+
+   private:
+      unsigned m_material;
+      Texture * m_oldTexture;
+      Texture * m_newTexture;
+};
+
 class MU_SetBackgroundImage : public ModelUndo
 {
    public:
@@ -1429,8 +1932,8 @@ class MU_SetBackgroundCenter : public ModelUndo
       unsigned size();
 
       void setBackgroundCenter( const unsigned & index, 
-            float newX, float newY, float newY,
-            float oldX, float oldY, float oldY );
+            float newX, float newY, float newZ,
+            float oldX, float oldY, float oldZ );
 
    private:
       unsigned m_index;
@@ -1458,6 +1961,27 @@ class MU_AddMetaData : public ModelUndo
       std::string m_value;
 };
 
+class MU_UpdateMetaData : public ModelUndo
+{
+   public:
+      MU_UpdateMetaData();
+      virtual ~MU_UpdateMetaData();
+
+      void undo( Model * );
+      void redo( Model * );
+      bool combine( Undo * );
+
+      unsigned size();
+
+      void updateMetaData( const std::string & key,
+            const std::string & newValue, const std::string & oldValue );
+
+   private:
+      std::string m_key;
+      std::string m_newValue;
+      std::string m_oldValue;
+};
+
 class MU_ClearMetaData : public ModelUndo
 {
    public:
